==> knt_ui-1.6.3.tar.gz: lib/src/widget/gallery/knt_photo_browser_page.dart <==
import 'package:flutter/material.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:flutter_staggered_grid_view/flutter_staggered_grid_view.dart';
import 'package:knt_bloc/knt_bloc.dart';

import '../../bloc/paging_gallery_bloc.dart';
import '../../config.dart';
import '../../defined.dart';
import '../../model/models.dart';
import '../image/knt_image.dart';
import '../knt_app_bar.dart';
import '../knt_issuing_widget.dart';
import '../knt_shimmer.dart';
import '../knt_status_bar_page.dart';
import 'knt_gallery_page.dart';

class KntPhotoBrowserPage extends StatelessWidget {
  const KntPhotoBrowserPage({
    Key? key,
    this.title,
  }) : super(key: key);

  final String? title;

  static void push(
    BuildContext context, {
    String? title,
  }) {
    Navigator.of(context).push(MaterialPageRoute(builder: (context) {
      return KntPhotoBrowserPage(
        title: title,
      );
    }));
  }

  @override
  Widget build(BuildContext context) {
    return KntBlackStatusBarPage(
      page: Scaffold(
        appBar: KntDefaultAppBar(
          titlePage: title,
        ),
        body: _PagingGallery(),
      ),
    );
  }
}

class _PagingGallery extends StatefulWidget {
  final double? scrollOffsetToLoadMore;

  const _PagingGallery({
    Key? key,
    this.scrollOffsetToLoadMore,
  }) : super(key: key);

  @override
  _PagingGalleryState createState() => _PagingGalleryState();
}

class _PagingGalleryState extends State<_PagingGallery> {
  late List<Photo> _photos;
  late bool _isLoading;

  @override
  void initState() {
    super.initState();
    _photos = [];
    _isLoading = false;
    Future.delayed(Duration.zero, _onFirstQuery);
  }

  void _onFirstQuery() {
    if (!mounted) {
      return;
    }
    context.read<PagingGalleryBloc>().add(BasePagingEvent<String?>(
          limit: DefinedUI.defaultFetchLimitGallery,
          isFirstRequest: true,
        ));
  }

  void _onLoadMoreResult() {
    if (!mounted) {
      return;
    }
    context.read<PagingGalleryBloc>().add(BasePagingEvent<String?>(
          limit: DefinedUI.defaultFetchLimitGallery,
        ));
  }

  void _onPickPhoto(BuildContext context, int index) {
    KntGalleryPage.push(context, photoList: List.from([_photos[index]]));
  }

  @override
  Widget build(BuildContext context) {
    return SizedBox(
      width: 1.sw,
      child: NotificationListener<ScrollNotification>(
        onNotification: (scrollInfo) {
          if (!_isLoading &&
              scrollInfo is ScrollUpdateNotification &&
              (scrollInfo.metrics.pixels - scrollInfo.metrics.maxScrollExtent >
                  (widget.scrollOffsetToLoadMore ?? 20.0))) {
            _onLoadMoreResult();
          }
          return false;
        },
        child: BlocBuilder<PagingGalleryBloc, BaseState>(
          builder: (context, state) {
            _isLoading = false;
            if (state is BaseErrorState) {
              return KntDefaultIssuingWidget.exception(
                onRetry: _onFirstQuery,
                exception: state.exception,
              );
            }
            if (state is BaseEmptyResultState) {
              return KntDefaultIssuingWidget();
            }
            if (state is BaseLoadingState) {
              _isLoading = true;
            }
            if (state is BasePagingResultState<Photo>) {
              _photos = state.data;
            }
            final storageType = _photos.isEmpty
                ? StorageType.network
                : _photos.first.storageType;
            final numberPhotoInBlock =
                DefinedUI.defaultPhotoCountInGalleryVertBlock;
            final borderRadius = BorderRadius.circular(16.w);
            return StaggeredGridView.countBuilder(
              shrinkWrap: true,
              crossAxisCount: DefinedUI.defaultPhotoCountInGalleryVertBlock - 1,
              physics: const BouncingScrollPhysics(),
              padding: EdgeInsets.symmetric(
                vertical: 16.w,
                horizontal: 24.w,
              ),
              itemCount: _photos.length,
              itemBuilder: (BuildContext context, int index) {
                if (index >= _photos.length) {
                  return KntShimmer(
                    borderRadius: borderRadius,
                    placeholder: Container(
                      color: KntUi.shared.shimmerColor,
                    ),
                  );
                }
                final photo = _photos[index];
                return InkWell(
                  onTap: () => _onPickPhoto(context, index),
                  child: Hero(
                    tag: photo.id,
                    child: KntImage(
                      uri: photo.mediumSizeUri,
                      borderRadius: borderRadius,
                      imageType: storageType,
                    ),
                  ),
                );
              },
              staggeredTileBuilder: (int index) {
                // Each bloc has [numberPhotoInBlock] photos which shared into 2 styles
                final bigChunk = index % numberPhotoInBlock == 0 ||
                    index % numberPhotoInBlock == (numberPhotoInBlock - 2);
                return StaggeredTile.count(bigChunk ? 3 : 2, bigChunk ? 4 : 2);
              },
              mainAxisSpacing: 12.w,
              crossAxisSpacing: 12.w,
            );
          },
        ),
      ),
    );
  }
}

==> knt_ui-1.6.4.tar.gz: lib/src/widget/gallery/knt_photo_browser_page.dart <==
import 'package:flutter/material.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:flutter_staggered_grid_view/flutter_staggered_grid_view.dart';
import 'package:knt_bloc/knt_bloc.dart';

import '../../bloc/paging_gallery_bloc.dart';
import '../../config.dart';
import '../../defined.dart';
import '../../model/models.dart';
import '../image/knt_image.dart';
import '../knt_app_bar.dart';
import '../knt_issuing_widget.dart';
import '../knt_shimmer.dart';
import '../knt_status_bar_page.dart';
import 'knt_gallery_page.dart';

class KntPhotoBrowserPage extends StatelessWidget {
  const KntPhotoBrowserPage({
    Key? key,
    this.title,
    this.scrollOffsetToLoadMore,
  }) : super(key: key);

  final String? title;
  final double? scrollOffsetToLoadMore;

  static void push(
    BuildContext context, {
    String? title,
    double? scrollOffsetToLoadMore,
  }) {
    Navigator.of(context).push(MaterialPageRoute(builder: (context) {
      return KntPhotoBrowserPage(
        title: title,
        scrollOffsetToLoadMore: scrollOffsetToLoadMore,
      );
    }));
  }

  @override
  Widget build(BuildContext context) {
    return KntBlackStatusBarPage(
      page: Scaffold(
        appBar: KntDefaultAppBar(
          titlePage: title,
        ),
        body: _PagingGallery(
          scrollOffsetToLoadMore: scrollOffsetToLoadMore,
        ),
      ),
    );
  }
}

class _PagingGallery extends StatefulWidget {
  const _PagingGallery({
    Key? key,
    this.scrollOffsetToLoadMore,
  }) : super(key: key);

  final double? scrollOffsetToLoadMore;

  @override
  _PagingGalleryState createState() => _PagingGalleryState();
}

class _PagingGalleryState extends State<_PagingGallery> {
  late List<Photo> _photos;
  late bool _isLoading;

  @override
  void initState() {
    super.initState();
    _photos = [];
    _isLoading = false;
    Future.delayed(Duration.zero, _onFirstQuery);
  }

  void _onFirstQuery() {
    if (!mounted) {
      return;
    }
    context.read<PagingGalleryBloc>().add(BasePagingEvent<String?>(
          limit: DefinedUI.defaultFetchLimitGallery,
          isFirstRequest: true,
        ));
  }

  void _onLoadMoreResult() {
    if (!mounted) {
      return;
    }
    context.read<PagingGalleryBloc>().add(BasePagingEvent<String?>(
          limit: DefinedUI.defaultFetchLimitGallery,
        ));
  }

  void _onPickPhoto(BuildContext context, int index) {
    KntGalleryPage.push(context, photoList: List.from([_photos[index]]));
  }

  @override
  Widget build(BuildContext context) {
    return SizedBox(
      width: 1.sw,
      child: NotificationListener<ScrollNotification>(
        onNotification: (scrollInfo) {
          if (!_isLoading &&
              scrollInfo is ScrollUpdateNotification &&
              (scrollInfo.metrics.pixels - scrollInfo.metrics.maxScrollExtent >
                  (widget.scrollOffsetToLoadMore ?? 20.0))) {
            _onLoadMoreResult();
          }
          return false;
        },
        child: BlocBuilder<PagingGalleryBloc, BaseState>(
          builder: (context, state) {
            _isLoading = false;
            if (state is BaseErrorState) {
              return KntDefaultIssuingWidget.exception(
                onRetry: _onFirstQuery,
                exception: state.exception,
              );
            }
            if (state is BaseEmptyResultState) {
              return KntDefaultIssuingWidget();
            }
            if (state is BaseLoadingState) {
              _isLoading = true;
            }
            if (state is BasePagingResultState<Photo>) {
              _photos = state.data;
            }
            final storageType = _photos.isEmpty
                ? StorageType.network
                : _photos.first.storageType;
            final numberPhotoInBlock =
                DefinedUI.defaultPhotoCountInGalleryVertBlock;
            final borderRadius = BorderRadius.circular(16.w);
            return StaggeredGridView.countBuilder(
              shrinkWrap: true,
              crossAxisCount: DefinedUI.defaultPhotoCountInGalleryVertBlock - 1,
              physics: const BouncingScrollPhysics(),
              padding: EdgeInsets.symmetric(
                vertical: 16.w,
                horizontal: 24.w,
              ),
              itemCount: _photos.length,
              itemBuilder: (BuildContext context, int index) {
                if (index >= _photos.length) {
                  return KntShimmer(
                    borderRadius: borderRadius,
                    placeholder: Container(
                      color: KntUi.shared.shimmerColor,
                    ),
                  );
                }
                final photo = _photos[index];
                return InkWell(
                  onTap: () => _onPickPhoto(context, index),
                  child: Hero(
                    tag: photo.id,
                    child: KntImage(
                      uri: photo.mediumSizeUri,
                      borderRadius: borderRadius,
                      imageType: storageType,
                    ),
                  ),
                );
              },
              staggeredTileBuilder: (int index) {
                // Each bloc has [numberPhotoInBlock] photos which shared into 2 styles
                final bigChunk = index % numberPhotoInBlock == 0 ||
                    index % numberPhotoInBlock == (numberPhotoInBlock - 2);
                return StaggeredTile.count(bigChunk ? 3 : 2, bigChunk ? 4 : 2);
              },
              mainAxisSpacing: 12.w,
              crossAxisSpacing: 12.w,
            );
          },
        ),
      ),
    );
  }
}
